import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutterupushplugin/register_info.dart';
import 'package:flutterupushplugin/u_push_options.dart';

typedef Future<dynamic> EventHandler(Map<String, dynamic> event);

class UpushPlugin {
  static const MethodChannel _channel =
      const MethodChannel('flutterupushplugin');
  static UpushPlugin _instance;

  factory UpushPlugin() => _getInstance();

  UpushPlugin._internal() {
    _channel.setMethodCallHandler(_handler);
  }

  static UpushPlugin _getInstance() {
    if (_instance == null) {
      _instance = UpushPlugin._internal();
    }
    return _instance;
  }

  void init(UPushOptions mUPushOptions) {
    _channel.invokeListMethod("init", mUPushOptions.toMap());
  }


  //自定义推送消息
  EventHandler onReceiveCustomMessage = null;

  //推送消息
  EventHandler onReceiveNotification = null;

  //在线通知栏的点击
  EventHandler onNotificatiClickHandler = null;

  //接受离线通知的点击
  EventHandler onReceiveOffLineNotification = null;

  void addEventHandle(
      {EventHandler onNotificatiClickHandler,
      EventHandler onReceiveCustomMessage,
      EventHandler onReceiveNotification,
        EventHandler onReceiveOffLineNotification}) {
    this.onNotificatiClickHandler = onNotificatiClickHandler;
    this.onReceiveCustomMessage = onReceiveCustomMessage;
    this.onReceiveNotification = onReceiveNotification;
    this.onReceiveOffLineNotification=onReceiveOffLineNotification;
  }

  /**
   * 添加标签
   */
  Future<Map<dynamic,dynamic>> addTags(List<String> list) async {
    final Map<dynamic,dynamic> result = await (_channel.invokeMethod("addTags", list)) ;

    debugPrint("************添加标签 addTags****${result}");

    return result;
  }

  /**
   * 删除标签
   */
  Future<Map<dynamic,dynamic>> deleteTags(List<String> list) async {
    final Map<dynamic,dynamic> result =
    await _channel.invokeMethod("deleteTags", list);
    return result;
  }


  /**
   * 获取标签
   */
  Future<List<dynamic>> getTags() async {
    final List<dynamic> result =
    await _channel.invokeMethod("getTags");
    return result;
  }


  Future<Map<dynamic,dynamic>> addAlias(String aliasId,String aliasType) async {
    final Map<dynamic,dynamic> result =
    await _channel.invokeMethod("addAlias",{"aliasId":aliasId,"aliasType":aliasType});
    return result;
  }

  Future<Map<dynamic,dynamic>> setAlias(String aliasId,String aliasType) async {
    final Map<dynamic,dynamic> result =
    await _channel.invokeMethod("setAlias",{"aliasId":aliasId,"aliasType":aliasType});
    return result;
  }
  Future<Map<dynamic,dynamic>> deleteAlias(String aliasId,String aliasType) async {
    final Map<dynamic,dynamic> result =
    await _channel.invokeMethod("deleteAlias",{"aliasId":aliasId,"aliasType":aliasType});
    return result;
  }

  /**
   * 注册id
   */
  Future<String> getRegistrationId() async {
    final String result =
    await _channel.invokeMethod("getRegistrationId");
    return result;
  }




  Future _handler(MethodCall call) {
    String method = call.method;
    Map map = call.arguments;
    debugPrint("************添加标签 map****${map}");
    if (method == "onNotificatiClickHandler") {
      //注册回调
      onNotificatiClickHandler(call.arguments.cast<String, dynamic>());
    } else if (method == "onReceiveCustomMessage") {
      //用户自定义消息
      onReceiveCustomMessage(call.arguments.cast<String, dynamic>());
    } else if (method == "onReceiveNotification") {
      //正常推送消息
      onReceiveNotification(call.arguments.cast<String, dynamic>());
    }else if(method=="onReceiveOffLineNotification"){
      //离线通知的点击
      debugPrint("**********enter **onOffLineMsgClickHandler ****");

      onReceiveOffLineNotification(call.arguments.cast<String, dynamic>());
    }


    else {
      throw new UnsupportedError("Unrecognized Event");
    }
  }
}
